<?php

namespace FiloBlu\ExtViSualMerchandiser\Helper;

use Exception;
use FiloBlu\ExtViSualMerchandiser\Model\StockStatus;
use FiloBlu\ExtViSualMerchandiser\Model\StockStatusInterface;
use FiloBlu\ExtViSualMerchandiser\Model\StockStatusInterfaceFactory;
use Magento\CatalogInventory\Api\StockRegistryInterface;
use Magento\CatalogInventory\Model\Stock;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Module\Manager;
use Magento\Framework\ObjectManagerInterface;

/**
 * Class StockHelper
 * @package FiloBlu\ExtViSualMerchandiser\Helper
 */
class StockHelper extends AbstractHelper
{

    /**
     * @var StockStatusInterface []
     */
    protected $stockStatus = [];
    /**
     * @var Manager
     */
    private $moduleManager;
    /**
     * @var ObjectManagerInterface
     */
    private $objectManager;
    /**
     * @var StockStatusInterfaceFactory
     */
    private $stockStatusFactory;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var ConfigHelper
     */
    private $configHelper;

    /**
     * StockHelper constructor.
     * @param Context $context
     * @param Manager $moduleManager
     * @param StockStatusInterfaceFactory $stockStatusFactory
     * @param ResourceConnection $resourceConnection
     * @param ConfigHelper $configHelper
     * @param ObjectManagerInterface $objectManager
     */
    public function __construct(
        Context $context,
        Manager $moduleManager,
        StockStatusInterfaceFactory $stockStatusFactory,
        ResourceConnection $resourceConnection,
        ConfigHelper $configHelper,
        ObjectManagerInterface $objectManager

    )
    {
        parent::__construct($context);
        $this->moduleManager = $moduleManager;
        $this->objectManager = $objectManager;
        $this->stockStatusFactory = $stockStatusFactory;
        $this->resourceConnection = $resourceConnection;
        $this->configHelper = $configHelper;
    }

    /**
     * @param $sku
     * @return StockStatusInterface[]
     */
    public function getStockStatus($sku)
    {
        if ($this->configHelper->isMultistockAvailable()) {

            $result = [];

            foreach ($this->getMsiStocks() as $stockId => $stockStatus) {
                try {
                    $this->stockStatus[$stockId]->setIsInStock($sku, $this->getMsiQuantity($sku, $stockId));
                    $result[$stockId] = $this->stockStatus[$stockId]->isInStock($sku);
                } catch (Exception $e) {
                    continue;
                }
            }

            return $result;
        }

        if (empty($this->stockStatus)) {
            $this->stockStatus[Stock::DEFAULT_STOCK_ID] = $this->objectManager->get(StockRegistryInterface::class);
        }

        /** @var StockRegistryInterface $stockStatus */
        $stockStatus = $this->stockStatus[Stock::DEFAULT_STOCK_ID];
        return [Stock::DEFAULT_STOCK_ID => $stockStatus->getStockStatusBySku($sku)->getStockStatus() === Stock::STOCK_IN_STOCK];
    }

    /**
     * @return StockStatusInterface[]
     */
    protected function getMsiStocks()
    {
        if (!empty($this->stockStatus)) {
            return $this->stockStatus;
        }

        $default = $this->configHelper->getSelectedDefaultStockId();
        $stocks = $this->configHelper->getAvailableStocks();

        foreach ($stocks as $stockId => $name) {
            /** @var StockStatusInterface $stockStatus */
            $stockStatus = $this->stockStatusFactory->create();
            $stockStatus->setId($stockId);
            $stockStatus->setName($name);
            $this->stockStatus[$stockId] = $stockStatus;
        }

        uksort($this->stockStatus, function ($a, $b) use ($default) {
            return ($a == $default || $b == $default);
        });
        return $this->stockStatus;
    }

    /**
     * @param string $sku
     * @param int $stockId
     * @return mixed
     */
    public function getMsiQuantity($sku, $stockId)
    {
        ///** @var \Magento\InventoryReservationsApi\Model\GetReservationsQuantityInterface $getReservationQuantity */
        //$getReservationQuantity = $this->objectManager->get(\Magento\InventoryReservationsApi\Model\GetReservationsQuantityInterface::class);
        $inventoryStock = $this->resourceConnection->getTableName("inventory_stock_{$stockId}");
        // TODO: manage backorder
        $row = $this->resourceConnection->getConnection()->fetchRow(
            "SELECT `is_salable` FROM `{$inventoryStock}` WHERE `sku` = :sku",
            ['sku' => $sku]
        );

        return (bool)$row['is_salable'];
    }

    /**
     * @param $stockId
     * @return string
     */
    public function getStockLabel($stockId)
    {
        if ((int)$stockId === Stock::DEFAULT_STOCK_ID) {
            return 'Default';
        }

        if (isset($this->stockStatus[$stockId])) {
            return $this->stockStatus[$stockId]->getName();
        }

        return null;
    }
}
